

;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
;:                i386 CPU MODES ROUTINES                         ::
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; RET EIP CAN'T BE MORE 0xffff

; SEG REGS = flatDS
; DESTROY: EAX, EDX
real2protected:
use16
	xor	edx, edx
	pop	dx		; real mode ip

	cli
; DISABLE NMI
	in	al, 70h
	or	al, 80h
	out	70h, al

; FAST OPEN A20 GATE
	in	al, 92h
	or	al, 2
	out	92h, al
pending_open:
	in	al, 92h
	test	al, 2
	jz	pending_open

; PROTECTED MODE ON
	mov	eax, cr0
	or	eax, 1
	mov	cr0, eax

	jmp	fword flatCS:pm
use32
pm:
	mov	ax, flatDS
	mov	ds, ax
	mov	es, ax
	mov	ss, ax
	; mov	esp, KERNSTACKTOP	; S.P. NOT MODIFY !

	lidt	fword [idtr]

; ENABLE NMI
	in	al, 0x70
	and	al, 7fh
	out	0x70, al

	sti

	push	edx
	ret


; SEG REGS = 0
; DESTROY: EAX, EDX
protected2real:
use32
	pop	edx

	cli
; DISABLE NMI
	in	al, 70h
	or	al, 80h
	out	70h, al

	jmp	fword realCS:rm
use16
rm:
	mov	eax, cr0
	and	eax, not 1
	mov	cr0, eax
	jmp	0:clear_prefetch
clear_prefetch:
	mov	ax, 0
	mov	ds, ax
	mov	es, ax
	mov	ss, ax
	; mov	sp, KERNSTACKTOP	; S.P. NOT MODIFY !

; FAST CLOSE A20 GATE
	in	al, 0x92
	and	al, not 2
	out	0x92, al
pending_close:
	in	al, 92h
	test	al, 2
	jnz	pending_close

	lidt	fword [idtr_rm]

; ENABLE NMI
	in	al, 0x70
	and	al, 7fh
	out	0x70, al

	sti

	push	dx
	ret
use32

;-----------------SYSTEM-DATA--------------------------

gdtr	dw gdt_tab_lim
	dd gdt

align 8
;	GLOBAL DESCRIPTOR TABLE
gdt:
	dw 0x0, 0x0				; NULL
	db 0x0, 0, 0, 0x0

	dw 0xffff, 0x0				; flatCS
	db 0x0, 10011010b, 11001111b, 0x0

	dw 0xffff, 0x0				; flatDS
	db 0x0, 10010010b, 11001111b, 0x0

	dw 0xffff, 0x0				; realCS
	db 0x0, 10011010b, 0, 0x0

gdt_tab_lim	= $ - gdt - 1


idtr	dw idt_tab_lim
	dd idt

idtr_rm	dw 0x3ff
	dd 0

align 8
;	INTERRUPT DESCRIPTOR TABLE
idt:
	times 8	dw intr, flatCS, 1000111000000000b, 0x0		; 0x0 - 0x7

		dw irq0_7, flatCS, 1000111000000000b, 0x0	; 0x8

		dw irq1_kbd, flatCS, 1000111000000000b, 0x0	; 0x9

	times 6	dw irq0_7, flatCS, 1000111000000000b, 0x0	; 0xa - 0xf

	times 97 dw intr, flatCS, 1000111000000000b, 0x0	; 0x10 - 0x6f

	times 8 dw irq8_15, flatCS, 1000111000000000b, 0x0	; 0x70 - 0x78

; SYSTEM CALLS (CAN'T USING NESTED INTERRUPTS)
;		dw intr_putc, flatCS, 1000111000000000b, 0x0	; 0x79
;
;		dw intr_puts, flatCS, 1000111000000000b, 0x0	; 0x7a
;
;		dw intr_getc, flatCS, 1000111000000000b, 0x0	; 0x7b
;
;		dw intr_gets, flatCS, 1000111000000000b, 0x0	; 0x7c
;
;		dw intr_outchar, flatCS, 1000111000000000b, 0x0	; 0x7d

idt_tab_lim = $ - idt - 1

align 8

;------------------------------------------------------

intr:
	iretd

;:::::::::IRQ:HANDLERS:::::::::::
irq0_7:
	push	eax

	mov	al, 0x20
	out	0x20, al

	pop	eax
	iretd

irq8_15:
	push	eax

	mov	al, 0x20
	out	0xa1, al

	pop	eax
	iretd

irq1_kbd:
	push	eax
	push	ecx

	in	al, 0x60		; read scan code of pressed key
	movzx	ecx, byte [KBD_BUFFPOS]
	mov	byte [KBDBUFF + ecx], al
	inc	byte [KBD_BUFFPOS]

  ; ACCEPT KEYBOARD INPUT
	in	al, 0x61
	or	al, 0x80
	out	0x61, al

  ; SEND EIO TO INTR CNTRL
	mov	al, 0x20
	out	0x20, al

	pop	ecx
	pop	eax
	iretd

; SYSTEM CALLS HANDLERS
;intr_putc:
;	call	putc
;	iretd
;
;intr_puts:
;	call	puts
;	iretd
;
;intr_getc:
;	sti
;	call	getc
;	iretd
;
;intr_gets:
;	sti
;	call	gets
;	iretd
;
;intr_outchar:
;	call	outchar
;	iretd

;--------STRINGS-----------------
about	db 'efer 0.1 pre-alpha (c) 2008 kflavour, ', 0
flatmod	db 'flat mode', 0xa, 0xa, 0

sysinf:
stcode	db 'start       ........', 0xa
endcode	db 'end         ........', 0xa
stk	db 'stack       ........', 0xa, 0xa
skip_si	= 12

db 'this operating system creating just for fun ', 1, 0xa
db 'and for research working Personal Computers', 0xa
db 'with architecture x86 IA-32', 0xa, 0xa, 0