/*
 * rdw - raw drive writer
 * (c) Copyright 2008 kflavour
 *
 * descr:  writing files on disk or files (starting with begin).
 * system: WinNT 5.1 or higher
 */

#define _WIN32_WINNT 0x0501
#include <stdio.h>
#include <windows.h>

#define INFO "rdw - raw drive writer by kflavour\nusage:\trdw \\\\.\\<drive> <src_filename>"
#define BUF_SIZE	1024 * 1024

DWORD NormOnSect(DWORD size);	// normalize on sector size (512)

HANDLE	hDrive, hFile;
DWORD	fwSize, nWritten, allWritten = 0, nReaded, allReaded = 0, nRet;
BYTE	*buf;
int		ans;
LARGE_INTEGER drivSize;

main(int argc, char *argv[])
{	// CHECK CMD LINE OPTIONS
	if (argc != 3)
	{
		printf(INFO);
		exit(-1);
	}

	// OPEN DRIVE
	if ((hDrive = CreateFile(	argv[1],
								GENERIC_READ | GENERIC_WRITE,
								FILE_SHARE_READ | FILE_SHARE_WRITE,
								NULL,
								OPEN_EXISTING,
								0,
								0 )) == INVALID_HANDLE_VALUE)
	{
		printf("can't open drive %s for write", argv[1]);
		exit(1);
	}


	// OPEN SRC FILE
	if ((hFile = CreateFile(	argv[2],
								GENERIC_READ,
								FILE_SHARE_READ | FILE_SHARE_WRITE,
								NULL,
								OPEN_EXISTING,
								0,
								0 )) == INVALID_HANDLE_VALUE)
	{
		printf("can't open file %s", argv[2]);
		exit(2);
	}

	// DRIVE SIZE >= SRC FILE SIZE ?
	DeviceIoControl(hDrive,
					IOCTL_DISK_GET_LENGTH_INFO,	// required NT 5.1 or higher
					NULL,
					0,
					&drivSize,
					sizeof(drivSize.QuadPart),
					&nRet,
					NULL	);

	if ( drivSize.QuadPart < (fwSize = NormOnSect(GetFileSize(hFile, NULL)))\
													&& drivSize.QuadPart != 0 )
	{
		printf("File %s is too big for write on drive, continue anyway? (Y/N)\n", argv[2]);
		if ((ans = getch()) != 'Y' && ans != 'y') exit(0);
	}


	if (! (buf = malloc(BUF_SIZE)) ) exit(4);

	// COMMIT WRITING
	printf("You really want write %s (%d bytes) to drive %s at offset 0? (Y/N)", argv[2], fwSize, argv[1]);
	if ((ans = getch()) != 'Y' && ans != 'y') exit(0);

	// WRITING
	printf("\nwriting");
	ZeroMemory(buf, BUF_SIZE);

	while (ReadFile(hFile, buf, BUF_SIZE, &nReaded, NULL))
	{
		if (!(nReaded)) break;
		if (nReaded < BUF_SIZE) nReaded = fwSize - allReaded;
		allReaded += nReaded;
		WriteFile(hDrive, buf, nReaded, &nWritten, NULL);
		if (nWritten == 0 || nReaded != nWritten) break;
		putchar('.');
		ZeroMemory(buf, BUF_SIZE);				// write zero if no readed data
		if ((allWritten += nWritten) == fwSize)	// all writed
		{
			CloseHandle(hDrive);
			printf("\ncomplete");
			exit(0);
		}
	}
	CloseHandle(hDrive);
	printf("\nERR: writed %d bytes", allWritten);	// not all writed
}

DWORD NormOnSect(DWORD size)
{
	while (size % 512) size++;
	return size;
}
